#include <iostream>
#include <fstream>
#include <vector>
#include <cassert>
#include <map>
#include <algorithm>
#include <array>

using namespace std;

static const int kModulo = 1000 * 1000 * 1000 + 7;

class Tree {
  public:
    Tree(int size):
        m_value(size),
        m_edges(size),
        m_need_down(size, {{0, 0}}),
        m_down(size, {{0, 0}}),
        m_up(size, {{0, 0}}),
        m_dp_down(size) {
    }

    void addEdge(int from, int to) {
        m_edges[from].push_back(to);
    }

    void setValue(int x, int v) {
        m_value[x] = v;
    }

    int value(int x) const {
        return m_value[x];
    }

    void needDownAt(int x, int v1, int v2) {
        m_need_down[x][0] = v1;
        m_need_down[x][1] = v2;
    }

    int down(int x, int index) const {
        return m_down[x][index];
    }

    int up(int x, int index) const {
        return m_up[x][index];
    }

    int size() const {
        return m_edges.size();
    }

    int total(int L) const {
        int answer = 0;
        for (int i = 0; i < size(); ++i)
            if (m_value[i] == L - 1)
                answer = (answer + m_up[i][0]) % kModulo;
        return answer;
    }

    void compute_up() {
        vector<int> ways;

        dfsUp(0, ways);
    }

    void compute_down(int L) {
        dfsDown(0, L);
    }

  private:
    void dfsUp(int node, vector<int> & ways) {
        for (int i = 0; i < 2; ++i) {
            int value = m_need_down[node][i];
            if (value >= int(ways.size()))
                ways.resize(value + 1, 0);
            if (value != -1) {
                if (value == 0)
                    m_up[node][i] = 1;
                else
                    m_up[node][i] = ways[value - 1];
                if (i == 1)
                    ways[value] = (ways[value] + m_up[node][i]) % kModulo;
            } else
                m_up[node][i] = 0;
        }

        for (auto &son : m_edges[node])
            dfsUp(son, ways);
        for (int i = 1; i < 2; ++i) {
            int value = m_need_down[node][i];
            if (value != -1) {
                ways[value] -= m_up[node][i];
                if (ways[value] < 0)
                    ways[value] += kModulo;
            }
        }
    }

    void dfsDown(int node, int L) {
        m_dp_down[node].clear();
        for (auto &son : m_edges[node]) {
            dfsDown(son, L);
            if (m_dp_down[son].size() > m_dp_down[node].size())
                swap(m_dp_down[son], m_dp_down[node]);
        }

        // the highest is already moved out, so we can calcualte now
        for (auto &son : m_edges[node]) {
            for (int i = 0; i < int(m_dp_down[son].size()); ++i)
                m_dp_down[node][i] = (m_dp_down[node][i] + m_dp_down[son][i]) % kModulo;
        }

        // let's answer the queries needed
        for (int i = 0; i < 2; ++i) {
            int position = L - m_need_down[node][i] - 2;
            if (m_need_down[node][i] == L - 1)
                m_down[node][i] = 1;
            else if (position < int(m_dp_down[node].size()))
                m_down[node][i] = m_dp_down[node][position];
            else
                m_down[node][i] = 0;
        }

        // and don't forget about ourself
        if (m_value[node] != -1) {
            int current = 0;
            int position = L - m_value[node] - 1;
            if (m_value[node] == L - 1) {
                current = 1;
            } else if (position - 1 < int(m_dp_down[node].size()))
                current = m_dp_down[node][position - 1];

            if (current > 0) {
                if (position == int(m_dp_down[node].size()))
                    m_dp_down[node].push_back(current);
                else
                    m_dp_down[node][position] = (m_dp_down[node][position] + current) % kModulo;
            }
        }
    }

    vector<int> m_value;

    vector< vector<int> > m_edges;

    vector<array<int, 2> > m_need_down;

    vector< array<int, 2> > m_down;
    vector< array<int, 2> > m_up;
    vector< vector<int> > m_dp_down;
};

int main() {
    ifstream cin("meow.in");
    ofstream cout("meow.out");

    int N, L, Q; assert(cin >> N >> L >> Q);
    assert(1 <= N && N <= 100000);
    assert(1 <= L && L <= N);
    assert(1 <= Q && Q <= 200000);
    Tree T(N);

    for (int i = 1; i < N; ++i) {
        int x; assert(cin >> x);
        assert(0 <= x && x < i);
        T.addEdge(x, i);
    }

    for (int i = 0; i < N; ++i) {
        int initial; assert(cin >> initial);
        assert(1 <= initial && initial <= L);
        --initial;
        T.setValue(i, initial);
    }

    int total_answer = 0;
    vector<int> query(Q);
    for (int i = 0; i < Q; ++i) {
        assert(cin >> query[i]);
        assert(1 <= query[i] && query[i] <= L);
        --query[i];
    }

    for (int i = 0; i < Q; i += N) {
        vector<int> query_answer(min(i + N, Q) - i, 0);
        // let's mark what we need down
        for (int j = 0; j < N; ++j)
            T.needDownAt(j, T.value(j), T.value(j));

        // just so we can get old answer
        T.compute_up();
        int answer = T.total(L);

        for (int j = i; j < i + N && j < Q; ++j) {
            T.needDownAt(j - i, /* old */T.value(j - i), /* new */query[j]);
        }


        T.compute_down(L);

        // let's modify and compute up 
        for (int j = i; j < i + N && j < Q; ++j) {
            T.setValue(j - i, query[j]);
        }

        // compute again
        T.compute_up();

        // decrease what we lost
        for (int j = i; j < i + N && j < Q; ++j) {
            query_answer[j - i] = (query_answer[j - i] - 1LL * T.down(j - i, 0) * T.up(j - i, 0)) % kModulo;
        }

        // increase what we gain
        for (int j = i; j < i + N && j < Q; ++j) {
            query_answer[j - i] = (query_answer[j - i] + 1LL * T.down(j - i, 1) * T.up(j - i, 1)) % kModulo;
        }

        for (int j = i; j < i + N && j < Q; ++j) {
            answer = (answer + query_answer[j - i]) % kModulo;
            if (answer < 0)
                answer += kModulo;
            total_answer = (total_answer + 1LL * (j + 1) * answer) % kModulo;
        }
    }

    cout << total_answer << "\n";
}
