//
// Problema meow
// meow O(Q * Wordlen)
// meow Mem(N * WordLen)
//
#include <algorithm>
#include <fstream>
#include <iostream>
#include <vector>
using namespace std;

const int kMaxN = 1e5 + 5, kMod = 1e9+7;

inline int& Mod(int& a) {
    if (a >= kMod) {
        return a -= kMod;
    } else if (a < 0) {
        return a += kMod;
    } else {
        return a;
    }
}

int word_len;

struct SuffixDp {
    vector<int> dp;

    void Reset() {
        dp.clear();
    }

    void Add(int label) {
        int invert_label = word_len - label;
        if (invert_label == 0) {
            if (dp.size() == 0) {
                dp.push_back(0);
            }
            Mod(dp[0] += 1);
        } else {
            if (invert_label == (int)dp.size()) {
                dp.push_back(dp.back());
            } else if (invert_label < (int)dp.size()) {
                Mod(dp[invert_label] += dp[invert_label - 1]);
            }
        }
    }

    int Get(int label) {
        int invert_label = word_len - label;
        if (invert_label < (int)dp.size()) {
            return dp[invert_label];
        } else {
            return 0;
        }
    }

    void Debug() {
        for (int i = (int)dp.size() - 1; i >= 0; i -= 1) {
            cerr << word_len - i << '\t' << dp[i] << '\n';
        }
        cerr << '\n';
    }
} suffix_dp[kMaxN];

int n;
vector<int> children[kMaxN];
int label[kMaxN];
int next_label[kMaxN];

int answer = 0;

int computed_down[kMaxN], hook[kMaxN];

int RenewDown() {
    for (int node = n - 1; node >= 0; node -= 1) {
        int mx = 0;
        int where = -1;

        for (auto itr : children[node]) {
            if ((int)suffix_dp[itr].dp.size() > mx) {
                mx = suffix_dp[itr].dp.size();
                where = itr;
            }
        }

        if (where == -1) {
            suffix_dp[node].Reset();
        } else {
            swap(suffix_dp[node], suffix_dp[where]);

            for (auto itr : children[node]) {
                if (itr == where) {
                    continue;
                }

                for (int j = 0; j < (int)suffix_dp[itr].dp.size(); j += 1) {
                    Mod(suffix_dp[node].dp[j] += suffix_dp[itr].dp[j]);
                }
            }
        }

        computed_down[node] = suffix_dp[node].Get(hook[node]);
        suffix_dp[node].Add(label[node]);   
//        cerr << "###" << node << '\t' << label[node] << '\n';
//        cerr << computed_down[node] << '\t' << hook[node] << '\n';
//        suffix_dp[node].Debug();       
    }

    return suffix_dp[0].Get(1);
}

int GetDown() {
    for (int i = 0; i < n; i += 1) {
        if (next_label[i] == word_len) {
            hook[i] = 0;
        } else {
            hook[i] = next_label[i] + 1;
        }
    }

    int ans = RenewDown();

//    cerr << "Computed down:\t";
    for (int i = 0; i < n; i += 1) {
        if (next_label[i] == word_len) {
            computed_down[i] = 1;
        }
//        cerr << computed_down[i] << '\t';
    }
//    cerr << "\n";
    
    return ans;
}

int added_value[kMaxN], before_down[kMaxN], now_down[kMaxN];
int solve_df_stack[kMaxN];

void SolveDf(int node) {
    added_value[node] = 0;
    Mod(added_value[node] -= 1LL * solve_df_stack[label[node] - 1] * before_down[node] % kMod);
    Mod(added_value[node] += 1LL * solve_df_stack[next_label[node] - 1] * now_down[node] % kMod);
    
//    cerr << "Added value:\t" << node << '\t' << added_value[node] << '\n';
//    cerr << "\t" << solve_df_stack[next_label[node] - 1] << '\t' << solve_df_stack[label[node] - 1] << '\n';
//    cerr << "\t" << before_down[node] << '\t' << now_down[node] << '\n';
  
    Mod(solve_df_stack[next_label[node]] += solve_df_stack[next_label[node] - 1]);

    for (auto itr : children[node]) {
        SolveDf(itr);
    }

    Mod(solve_df_stack[next_label[node]] -= solve_df_stack[next_label[node] - 1]);
}

void Solve() {
    solve_df_stack[0] = 1;
    SolveDf(0);
}

int main() {
    ifstream cin("meow.in");
    ofstream cout("meow.out");

    int q;
    cin >> n >> word_len >> q;

    for (int i = 1; i < n; i += 1) {
        int x; cin >> x;
        children[x].push_back(i);
    }
    
    for (int i = 0; i < n; i += 1) {
        cin >> label[i];
    }

    for (int i = 0; i < n; i += 1) {
        next_label[i] = label[i];
    }
    
//    answer = GetDown();

    int sum = 0;
    for (int i = 0; i < q; i += n) {    
        answer = GetDown();

        for (int i = 0; i < n; i += 1) {
            before_down[i] = computed_down[i];
        }
    
        for (int j = 0; i + j < q and j < n; j += 1) {
            int l; cin >> l;
            next_label[j] = l;
        }

        GetDown();

        for (int i = 0; i < n; i += 1) {
            now_down[i] = computed_down[i];
        }

        Solve();

        for (int j = 0; j < n; j += 1) {
            label[j] = next_label[j];
        }       

        for (int j = 0; i + j < q and j < n; j += 1) {
            Mod(answer += added_value[j]);
            Mod(sum += 1LL * (i + 1 + j) * answer % kMod);
            //cerr << answer << '\n';
        }
    }
    cout << sum << '\n';

    return 0;
}
