#include <iostream>
#include <fstream>
#include <vector>

using namespace std;

class ZP {
  public:
    explicit ZP(const int &x = 0): m_data(x % kModulo) {
    }

    explicit ZP(const int64_t &x): m_data(x % kModulo) {
    }

    ZP& operator+=(const ZP& that) {
        m_data += that.m_data;
        if (m_data >= kModulo)
            m_data -= kModulo;
        if (m_data <= -kModulo)
            m_data += kModulo;
        return *this;
    }

    ZP& operator-=(const ZP& that) {
        *this = ZP{m_data - that.m_data};
        return *this;
    }

    ZP operator*(const ZP& that) const {
        return ZP{int64_t(m_data) * that.m_data};
    }

    operator int() const {
        if (m_data < 0)
            return m_data + kModulo;
        return m_data;
    }

    static const int kModulo = 1000 * 1000 * 1000 + 7;

    int m_data;
};

static const int kMaxN = 100;
static const int kMaxT = 202;
static const int kMaxK = 200;

ZP comb[2 * kMaxN + 1][2 * kMaxN + 1];
ZP dp[kMaxN + 1][kMaxT + 1][kMaxT + 1];
ZP dp2[kMaxT + 1][kMaxT + 1][kMaxK + 1];

int main() {
    ifstream cin("piruete.in");
    ofstream cout("piruete.out");

    int T, N, K; cin >> T >> N >> K;

    comb[0][0] = ZP{1};
    for (int i = 0; i < 2 * N; ++i)
        for (int j = 0; j <= i; ++j) {
            comb[i + 1][j] += comb[i][j];
            comb[i + 1][j + 1] += comb[i][j];
        }

    if (T == 0) {
        cout << comb[2 * N][K] << "\n";
        return 0;
    }

    if (T % 2) {
        cout << 0 << "\n";
        return 0;
    }
    T /= 2;

    // dp[i][j][k] = have until wall i, exactly j walls and sum k
    // but if we put j walls at 1, 2, 3, ..., j
    // we get at least the sum (1 + 2  + 3 + ... + j) = j * (j + 1) / 2
    int max_j;
    for (max_j = 1; (max_j + 1) * (max_j + 2) / 2 <= T; ++max_j);

    dp[0][0][0] = ZP{1};
    for (int i = 1; i <= N; ++i)
        for (int j = 0; j <= max_j; ++j)
            for (int k = 0; k <= T; ++k) {
                dp[i][j][k] = dp[i - 1][j][k];
                if (k >= i && j > 0)
                    dp[i][j][k] += dp[i - 1][j - 1][k - i];
            }
    // let's make dp[i][j][k] = have until wall i, exactly j walls and sum k and really a wall at i
    for (int i = N; i >= 1; --i)
        for (int j = 0; j <= max_j; ++j)
            for (int k = 0; k <= T; ++k)
                dp[i][j][k] -= dp[i - 1][j][k];

    // dp2[i][j][k] = have exactly i wall-hits, sum j and k created walls that we do not hit
    //                but so far without ever hitting N + 1 wall
    for (int i = 0; i <= N; ++i)
        for (int j = 0; j <= max_j && j <= K; ++j)
            for (int k = 0; k <= T; ++k) {
                // first with 0 extra walls
                dp2[j][k][j] += dp[i][j][k];
            }

    // then still 0 extra walls, but with wall-hits at N + 1
    for (int i = 1; i <= T; ++i)
        for (int j = N + 1; j <= T; ++j)
            for (int k = 0; k <= K; ++k)
                dp2[i][j][k] += dp2[i - 1][j - N - 1][k];

    // then the rest, with extra walls
    for (int i = 0; i <= N; ++i)
        for (int j = 0; j <= max_j && j <= i; ++j)
            for (int k = j * (j + 1) / 2; k <= T; ++k)
                for (int l = 1; j + l <= K; ++l) {
                    dp2[j][k][j + l].m_data = (1LL * dp[i][j][k].m_data * comb[N - i][l].m_data + dp2[j][k][j + l].m_data) % ZP::kModulo;
                }
    // combine and get answer
    ZP answer{0};
    for (int wall_hits_left = 0; wall_hits_left <= T; ++wall_hits_left)
        for (int wall_hits_right = wall_hits_left; wall_hits_right <= min(wall_hits_left + 1, T); ++wall_hits_right)
            for (int created_left = 0; created_left <= K; ++created_left) {
                int created_right = K - created_left;
                for (int sum_left = 0; sum_left <= T; ++sum_left) {
                    int sum_right = T - sum_left;
                    answer += dp2[wall_hits_left][sum_left][created_left] * dp2[wall_hits_right][sum_right][created_right];
                }
            }


    cout << answer << "\n";
}