/* O(N * M * logN) */
#include <bits/stdc++.h>

using namespace std;

struct Range {
	int l, r, w;
};

struct SegmentTree {
	vector<int> L, T;
	int n;

	SegmentTree(int n) : L(4 * n), T(4 * n), n(n) {}

	void increment(int node, int b, int e, int l, int r, int val) {
		if(b > r || e < l) return;

		if(b >= l && e <= r) {
			L[node] += val;
			T[node] += val;
		} else {
			int m = (b + e) / 2;
			increment(node * 2, b, m, l, r, val);
			increment(node * 2 + 1, m + 1, e, l, r, val);
			T[node] = max(T[node * 2], T[node * 2 + 1]) + L[node];
		}
	}
	void Increment(int l, int r, int val) {
		increment(1, 0, n - 1, l, r, val);
	}

	int QueryMin() {
		return T[1];
	}
};

vector<int> SolveStep(vector<int> old, vector<Range> &ranges) {
	int n = old.size();
	vector<int> ret(n);
	SegmentTree st(n);

	int cursor = 0;
	for(int e = 0; e < n; ++e) {
		if(e > 0) st.Increment(e, e, old[e - 1]);
		while(cursor < ranges.size()) {
			auto &range = ranges[cursor];
			if(range.r == e) {
				st.Increment(range.l, e, range.w);
				++cursor;
			} else break;
		}
		ret[e] = st.QueryMin();
	}

	return ret;
}

int main() {
    ifstream cin("popcorn.in");
    ofstream cout("popcorn.out");
	int n, m;
	cin >> n >> m;
	
	vector<Range> ranges(n);
	for (auto &range : ranges) {
		cin >> range.l >> range.r >> range.w;
		range.r -= 1;
	}
	
	sort(ranges.begin(), ranges.end(), [](Range a, Range b) {
		return a.r < b.r;
	});

	int maxx = ranges.back().r;

	vector<int> dp(maxx + 1, 0);
	for (int i = 0; i < m; ++i)
		dp = SolveStep(dp, ranges);

	cout << dp.back() << endl;

	return 0;
}
