/* O(N * logVALMAX * log N) */
#include <bits/stdc++.h>

using namespace std;

struct Range {
    int l, r, w;
};

struct SegmentTree {
    vector<double> L;
    vector<pair<double, int>> T;
    int n;

    SegmentTree(int n) : L(4 * n), T(4 * n), n(n) {}

    void increment(int node, int b, int e, int l, int r, double val) {
        if(b > r || e < l) return;

        if(b >= l && e <= r) {
            L[node] += val;
            T[node].first += val;
        } else {
            int m = (b + e) / 2;
            increment(node * 2, b, m, l, r, val);
            increment(node * 2 + 1, m + 1, e, l, r, val);
            T[node] = max(T[node * 2], T[node * 2 + 1]);
            T[node].first += L[node];
        }
    }
    void Increment(int l, int r, double val) {
        increment(1, 0, n - 1, l, r, val);
    }

    void set(int node, int b, int e, int pos, pair<double, int> val) {
        if (b == e) T[node] = val;
        else {
            int m = (b + e) / 2;
            if (m >= pos) set(node * 2, b, m, pos, val);
            else set(node * 2 + 1, m + 1, e, pos, val);
            
            T[node] = max(T[node * 2], T[node * 2 + 1]);
            T[node].first += L[node];
        }
    }
    void Set(int pos, pair<double, int> val) {
        set(1, 0, n - 1, pos, val);
    }

    pair<double, int> QueryMax() {
        return T[1];
    }
};

pair<double, int> Evaluate(double m, vector<Range> &ranges) {
    int n = ranges.back().r + 1;
    SegmentTree st(n);

    int cursor = 0;
    for(int e = 0; e < n; ++e) {
        auto p = st.QueryMax();
        p.second += 1; p.first -= m;

        st.Set(e, p);

        while(cursor < ranges.size()) {
            auto &range = ranges[cursor];
            if(range.r == e) {
                st.Increment(range.l, e, range.w);
                ++cursor;
            } else break;
        }
    }

    return st.QueryMax();
}

int main() {
    ifstream cin("popcorn.in");
    ofstream cout("popcorn.out");
    
    int n, k;
    cin >> n >> k;
    
    vector<Range> ranges(n);
    for (auto &range : ranges) {
        cin >> range.l >> range.r >> range.w;
        range.r -= 1;
    }
    
    sort(ranges.begin(), ranges.end(), [](Range a, Range b) {
        return a.r < b.r;
    });

    double b = 0, e = 1e9;
    for (int it = 0; it < 64; ++it) {
        double m = (b + e) / 2;
        auto p = Evaluate(m, ranges);
        if (p.second == k) break;
        if (p.second < k) e = m;
        else b = m;
    }


    double m = (b + e) / 2;
    auto p = Evaluate(m, ranges);
    cout << int(round(p.first + m * k)) << endl;

    return 0;
}
